# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: compute_sql.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11\x63ompute_sql.proto\x12\x0b\x63ompute_sql\"\xa3\x01\n\x16SqlWorkerConfiguration\x12:\n\nvalidation\x18\x01 \x01(\x0b\x32$.compute_sql.ValidationConfigurationH\x00\x12<\n\x0b\x63omputation\x18\x02 \x01(\x0b\x32%.compute_sql.ComputationConfigurationH\x00\x42\x0f\n\rconfiguration\"H\n\x17ValidationConfiguration\x12-\n\x0btableSchema\x18\x01 \x01(\x0b\x32\x18.compute_sql.TableSchema\"=\n\x0bTableSchema\x12.\n\x0cnamedColumns\x18\x01 \x03(\x0b\x32\x18.compute_sql.NamedColumn\"V\n\x0bNamedColumn\x12\x11\n\x04name\x18\x01 \x01(\tH\x00\x88\x01\x01\x12+\n\ncolumnType\x18\x02 \x01(\x0b\x32\x17.compute_sql.ColumnTypeB\x07\n\x05_name\"Q\n\nColumnType\x12\x31\n\rprimitiveType\x18\x01 \x01(\x0e\x32\x1a.compute_sql.PrimitiveType\x12\x10\n\x08nullable\x18\x02 \x01(\x08\";\n\x16TableDependencyMapping\x12\r\n\x05table\x18\x01 \x01(\t\x12\x12\n\ndependency\x18\x02 \x01(\t\"\xdb\x01\n\x18\x43omputationConfiguration\x12\x14\n\x0csqlStatement\x18\x01 \x01(\t\x12\x35\n\x0fprivacySettings\x18\x02 \x01(\x0b\x32\x1c.compute_sql.PrivacySettings\x12,\n\x0b\x63onstraints\x18\x03 \x03(\x0b\x32\x17.compute_sql.Constraint\x12\x44\n\x17tableDependencyMappings\x18\x04 \x03(\x0b\x32#.compute_sql.TableDependencyMapping\"2\n\x0fPrivacySettings\x12\x1f\n\x17minAggregationGroupSize\x18\x01 \x01(\x03\"!\n\nConstraint\x12\x13\n\x0b\x64\x65scription\x18\x01 \x01(\t*3\n\rPrimitiveType\x12\t\n\x05INT64\x10\x00\x12\n\n\x06STRING\x10\x01\x12\x0b\n\x07\x46LOAT64\x10\x02\x62\x06proto3')

_PRIMITIVETYPE = DESCRIPTOR.enum_types_by_name['PrimitiveType']
PrimitiveType = enum_type_wrapper.EnumTypeWrapper(_PRIMITIVETYPE)
INT64 = 0
STRING = 1
FLOAT64 = 2


_SQLWORKERCONFIGURATION = DESCRIPTOR.message_types_by_name['SqlWorkerConfiguration']
_VALIDATIONCONFIGURATION = DESCRIPTOR.message_types_by_name['ValidationConfiguration']
_TABLESCHEMA = DESCRIPTOR.message_types_by_name['TableSchema']
_NAMEDCOLUMN = DESCRIPTOR.message_types_by_name['NamedColumn']
_COLUMNTYPE = DESCRIPTOR.message_types_by_name['ColumnType']
_TABLEDEPENDENCYMAPPING = DESCRIPTOR.message_types_by_name['TableDependencyMapping']
_COMPUTATIONCONFIGURATION = DESCRIPTOR.message_types_by_name['ComputationConfiguration']
_PRIVACYSETTINGS = DESCRIPTOR.message_types_by_name['PrivacySettings']
_CONSTRAINT = DESCRIPTOR.message_types_by_name['Constraint']
SqlWorkerConfiguration = _reflection.GeneratedProtocolMessageType('SqlWorkerConfiguration', (_message.Message,), {
  'DESCRIPTOR' : _SQLWORKERCONFIGURATION,
  '__module__' : 'compute_sql_pb2'
  # @@protoc_insertion_point(class_scope:compute_sql.SqlWorkerConfiguration)
  })
_sym_db.RegisterMessage(SqlWorkerConfiguration)

ValidationConfiguration = _reflection.GeneratedProtocolMessageType('ValidationConfiguration', (_message.Message,), {
  'DESCRIPTOR' : _VALIDATIONCONFIGURATION,
  '__module__' : 'compute_sql_pb2'
  # @@protoc_insertion_point(class_scope:compute_sql.ValidationConfiguration)
  })
_sym_db.RegisterMessage(ValidationConfiguration)

TableSchema = _reflection.GeneratedProtocolMessageType('TableSchema', (_message.Message,), {
  'DESCRIPTOR' : _TABLESCHEMA,
  '__module__' : 'compute_sql_pb2'
  # @@protoc_insertion_point(class_scope:compute_sql.TableSchema)
  })
_sym_db.RegisterMessage(TableSchema)

NamedColumn = _reflection.GeneratedProtocolMessageType('NamedColumn', (_message.Message,), {
  'DESCRIPTOR' : _NAMEDCOLUMN,
  '__module__' : 'compute_sql_pb2'
  # @@protoc_insertion_point(class_scope:compute_sql.NamedColumn)
  })
_sym_db.RegisterMessage(NamedColumn)

ColumnType = _reflection.GeneratedProtocolMessageType('ColumnType', (_message.Message,), {
  'DESCRIPTOR' : _COLUMNTYPE,
  '__module__' : 'compute_sql_pb2'
  # @@protoc_insertion_point(class_scope:compute_sql.ColumnType)
  })
_sym_db.RegisterMessage(ColumnType)

TableDependencyMapping = _reflection.GeneratedProtocolMessageType('TableDependencyMapping', (_message.Message,), {
  'DESCRIPTOR' : _TABLEDEPENDENCYMAPPING,
  '__module__' : 'compute_sql_pb2'
  # @@protoc_insertion_point(class_scope:compute_sql.TableDependencyMapping)
  })
_sym_db.RegisterMessage(TableDependencyMapping)

ComputationConfiguration = _reflection.GeneratedProtocolMessageType('ComputationConfiguration', (_message.Message,), {
  'DESCRIPTOR' : _COMPUTATIONCONFIGURATION,
  '__module__' : 'compute_sql_pb2'
  # @@protoc_insertion_point(class_scope:compute_sql.ComputationConfiguration)
  })
_sym_db.RegisterMessage(ComputationConfiguration)

PrivacySettings = _reflection.GeneratedProtocolMessageType('PrivacySettings', (_message.Message,), {
  'DESCRIPTOR' : _PRIVACYSETTINGS,
  '__module__' : 'compute_sql_pb2'
  # @@protoc_insertion_point(class_scope:compute_sql.PrivacySettings)
  })
_sym_db.RegisterMessage(PrivacySettings)

Constraint = _reflection.GeneratedProtocolMessageType('Constraint', (_message.Message,), {
  'DESCRIPTOR' : _CONSTRAINT,
  '__module__' : 'compute_sql_pb2'
  # @@protoc_insertion_point(class_scope:compute_sql.Constraint)
  })
_sym_db.RegisterMessage(Constraint)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _PRIMITIVETYPE._serialized_start=878
  _PRIMITIVETYPE._serialized_end=929
  _SQLWORKERCONFIGURATION._serialized_start=35
  _SQLWORKERCONFIGURATION._serialized_end=198
  _VALIDATIONCONFIGURATION._serialized_start=200
  _VALIDATIONCONFIGURATION._serialized_end=272
  _TABLESCHEMA._serialized_start=274
  _TABLESCHEMA._serialized_end=335
  _NAMEDCOLUMN._serialized_start=337
  _NAMEDCOLUMN._serialized_end=423
  _COLUMNTYPE._serialized_start=425
  _COLUMNTYPE._serialized_end=506
  _TABLEDEPENDENCYMAPPING._serialized_start=508
  _TABLEDEPENDENCYMAPPING._serialized_end=567
  _COMPUTATIONCONFIGURATION._serialized_start=570
  _COMPUTATIONCONFIGURATION._serialized_end=789
  _PRIVACYSETTINGS._serialized_start=791
  _PRIVACYSETTINGS._serialized_end=841
  _CONSTRAINT._serialized_start=843
  _CONSTRAINT._serialized_end=876
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

import base64
import functools
import uuid
from enum import Enum
from typing import TYPE_CHECKING, Dict, List, Optional

from typing_extensions import Self

from ..attestation import enclave_specifications
from ..proto.length_delimited import serialize_length_delimited
from ..session import LATEST_WORKER_PROTOCOL_VERSION
from ..types import EnclaveSpecification
from .analytics_dcr import AnalyticsDcrDefinition
from .node_definitions import NodeDefinition
from .version import DATA_SCIENCE_DCR_SUPPORTED_VERSION

if TYPE_CHECKING:
    from ..client import Client


__all__ = [
    "AnalyticsDcrBuilder",
]


class AnalyticsDcrType(str, Enum):
    STATIC = "STATIC"
    INTERACTIVE = "INTERACTIVE"


class ParticipantPermission(Enum):
    DATA_OWNER = 1
    ANALYST = 2


def _get_hl_specs(enclave_specs: Dict[str, EnclaveSpecification]):
    specs = [
        {
            "attestationProtoBase64": base64.b64encode(
                serialize_length_delimited(spec["proto"])
            ).decode(),
            "id": name,
            "workerProtocol": spec.get(
                "workerProtocol", LATEST_WORKER_PROTOCOL_VERSION
            ),
        }
        for name, spec in enclave_specs.items()
    ]
    return specs


class AnalyticsDcrBuilder:
    """
    A builder for constructing Analytics Data Clean Rooms.
    """

    def __init__(
        self,
        *,
        client: Client,
        enclave_specs: Optional[Dict[str, EnclaveSpecification]] = None,
    ) -> None:
        """
        Initialise an Analytics DCR builder.

        **Parameters**:
        - `client`: A `Client` object that can be used to retrieve information about the platform.
        - `enclave_specs`: Determines the types of enclaves that will supported by this Data Clean Room.
            If not specified, the latest enclave specifications known to this
            SDK version will be used.
        """
        self.client = client
        self.enclave_specs = (
            enclave_specs if enclave_specs else enclave_specifications.latest()
        )
        self.name = None
        self.description = ""
        self.owner = None
        self.dcr_id = None
        self.compile_context = None
        self.node_definitions = []
        """The current list of Node Definitions that will be added to the Data Clean Room."""
        self.permissions = []
        """The list of permissions that will be added to the Data Clean Room."""

    def with_name(self, name: str) -> Self:
        """
        Set the name of the Data Clean Room.

        **Parameters**:
        - `name`: Name to be used for the Data Clean Room.
        """
        self.name = name
        return self

    def with_description(self, description: str) -> Self:
        """
        Set the description of the Data Clean Room.

        **Parameters**:
        - `description`: Description of the Data Clean Room.
        """
        self.description = description
        return self

    def add_participant(
        self,
        email: str,
        *,
        analyst_of: List[str] = [],
        data_owner_of: List[str] = [],
    ) -> Self:
        """
        Add a participant to the DCR being built.

        If the participant isn't assigned a role, the user can still view the
        DCR but cannot interact with it.

        **Parameters**:
        - `email`: The email address of the participant.
        - `analyst_of`: The names of the Compute Nodes that the user can run.
        - `data_owner_of`: The names of the Data Nodes to which the user can
          connect a dataset.
        """
        if any(permissions["user"] == email for permissions in self.permissions):
            raise Exception(f"Participant with email {email} has already been added.")

        anaylst_permissions = [{"analyst": {"nodeId": node}} for node in analyst_of]
        data_owner_permissions = [
            {"dataOwner": {"nodeId": node}} for node in data_owner_of
        ]
        self.permissions.append(
            {"user": email, "permissions": anaylst_permissions + data_owner_permissions}
        )
        return self

    def with_owner(self, email: str) -> Self:
        """
        Set the owner of the Data Clean Room.

        **Parameters**:
        - `email`: The email address of the owner of the Data Clean Room.
        """
        self.owner = email
        return self

    def add_node_definition(self, definition: NodeDefinition) -> Self:
        """
        Add a single node definition to this builder.

        A node definition defines how a Compute or Data Node
        should be constructed.
        """
        self.add_node_definitions([definition])
        return self

    def add_node_definitions(self, definitions: List[NodeDefinition]) -> Self:
        """
        Add a list of node definitions to this builder.

        Each node definition defines how the respective Compute or Data Node
        should be constructed.
        """
        self.node_definitions.extend(definitions)
        return self

    def build(self) -> AnalyticsDcrDefinition:
        """
        Build the Data Clean Room.

        In order to use the DCR, the output of this method should be passed to
        `client.publish_analytics_dcr`.
        """
        if not self.owner:
            raise Exception("The Data Room owner must be specified")
        if not self.name:
            raise Exception("The Data Room name must be specified")

        nodes = [
            node._get_high_level_representation() for node in self.node_definitions
        ]
        required_workers = functools.reduce(
            lambda a, b: a.union(b),
            [set(node.required_workers) for node in self.node_definitions],
            {"decentriq.driver"},
        )
        used_enclave_specs = {}
        for worker in required_workers:
            if worker in self.enclave_specs:
                used_enclave_specs[worker] = self.enclave_specs[worker]
            else:
                raise Exception(
                    f"One of the nodes you added requires a worker of type '{worker}',"
                    " but no enclave specification matching this worker is known to this builder."
                )
        used_hl_enclave_specs = _get_hl_specs(used_enclave_specs)
        permissions = self._add_owner_permissions()
        hl_dcr = {
            DATA_SCIENCE_DCR_SUPPORTED_VERSION: {
                # Only interactive DCRs are supported.
                "interactive": {
                    "commits": [],
                    "enableAutomergeFeature": True,
                    "initialConfiguration": {
                        "description": self.description,
                        "enableAirlock": True,
                        "enableAllowEmptyFilesInValidation": True,
                        "enableDevelopment": True,
                        "enablePostWorker": True,
                        "enableSafePythonWorkerStacktrace": True,
                        "enableServersideWasmValidation": True,
                        "enableSqliteWorker": True,
                        "enableTestDatasets": True,
                        "enclaveRootCertificatePem": self.client.decentriq_ca_root_certificate.decode(),
                        "enclaveSpecifications": used_hl_enclave_specs,
                        "id": self._generate_id(),
                        "nodes": nodes,
                        "participants": permissions,
                        "title": self.name,
                    },
                }
            }
        }
        return AnalyticsDcrDefinition(
            name=self.name, high_level=hl_dcr, enclave_specs=self.enclave_specs
        )

    def _add_owner_permissions(self):
        for entry in self.permissions:
            if entry["user"] == self.owner:
                permissions = entry["permissions"]
                permissions.append({"manager": {}})
                entry["permissions"] = permissions
                return self.permissions

        # Entry wasn't found for existing user, so add a new one.
        self.permissions.append({"user": self.owner, "permissions": [{"manager": {}}]})
        return self.permissions

    @staticmethod
    def _generate_id():
        return str(uuid.uuid4())



from typing import List
from decentriq_dcr_compiler import compiler
from decentriq_dcr_compiler.schemas import MediaInsightsRequest, MediaInsightsResponse

from ..channel import Channel
from ..proto import serialize_length_delimited
from ..session import LATEST_GCG_PROTOCOL_VERSION, Session

class Request:
    @staticmethod
    def send(
        request: MediaInsightsRequest,
        session: Session
    ) -> MediaInsightsResponse:
        def compile_request(mi_request: MediaInsightsRequest, channel: Channel):
            user_auth = channel._get_message_auth(session.auth)
            request_serialized = compiler.compile_media_insights_request(
                mi_request,
                serialize_length_delimited(user_auth),
            )
            return bytes(request_serialized)

        def decompile_response(responses: List[bytes]) -> MediaInsightsResponse:
            if len(responses) != 1:
                raise Exception("Malformed response")
            response = compiler.decompile_media_insights_response(
                request, bytes(responses[0])
            )
            return response

        # TODO: The `endpoint_protocols` should come from DDC as it knows
        # the appropriate supported versions.
        endpoint_protocols = [3, 4, 5, 6]
        protocol = session._get_client_protocol(endpoint_protocols)
        response = session.send_compilable_request(
            compile_request, request, decompile_response, protocol 
        )
        return response




from dataclasses import dataclass
from enum import Enum
from typing import Optional, Dict, Any


class ActivationType(str, Enum):
    LOOKALIKE = "lookalike"
    RETARGET = "retarget"


@dataclass
class Audience:
    audience_type: str
    activation_type: ActivationType
    is_published: bool
    # `reach` only required when `activation_type` is lookalike.
    reach: Optional[int] = None
    exclude_seed_audience: bool = False

    def as_dict(self) -> Dict[str, Any]:
        if self.reach:
            return {
                "audience_type": self.audience_type,
                "activation_type": self.activation_type.value,
                "is_published": self.is_published,
                "reach": self.reach,
                "exclude_seed_audience": self.exclude_seed_audience,
            }
        else:
            return {
                "audience_type": self.audience_type,
                "activation_type": self.activation_type.value,
                "is_published": self.is_published,
                "exclude_seed_audience": False, # Can only be set for Lookalike activateion types.
            }



from .attestation_pb2 import (
    AttestationSpecification,
    AttestationSpecificationAmdSnp,
    AttestationSpecificationAwsNitro,
    AttestationSpecificationIntelDcap,
    AttestationSpecificationIntelDcapMrsigner,
    AttestationSpecificationIntelEpid,
    Fatquote,
    FatquoteDcap,
    FatquoteEpid,
)
from .azure_blob_storage_pb2 import AzureBlobStorageWorkerConfiguration
from .compute_container_pb2 import ContainerWorkerConfiguration
from .compute_post_pb2 import PostWorkerConfiguration
from .compute_s3_sink_pb2 import S3SinkWorkerConfiguration
from .compute_sql_pb2 import SqlWorkerConfiguration
from .data_room_pb2 import (
    AddModification,
    AffectedDataOwnersApprovePolicy,
    AuthenticationMethod,
    ChangeModification,
    ComputeNode,
    ComputeNodeAirlock,
    ComputeNodeBranch,
    ComputeNodeFormat,
    ComputeNodeLeaf,
    ComputeNodeParameter,
    ComputeNodeProtocol,
    ConfigurationCommit,
    ConfigurationElement,
    ConfigurationModification,
    DataRoom,
    DataRoomConfiguration,
    DcrSecretPolicy,
    DryRunPermission,
    ExecuteComputePermission,
    ExecuteDevelopmentComputePermission,
    GenerateMergeSignaturePermission,
    GovernanceProtocol,
    LeafCrudPermission,
    MergeConfigurationCommitPermission,
    Permission,
    PkiPolicy,
    RetrieveAuditLogPermission,
    RetrieveComputeResultPermission,
    RetrieveDataRoomPermission,
    RetrieveDataRoomStatusPermission,
    RetrievePublishedDatasetsPermission,
    StaticDataRoomPolicy,
    UpdateDataRoomStatusPermission,
    UserPermission,
)
from .data_source_s3_pb2 import DataSourceS3WorkerConfiguration
from .data_source_snowflake_pb2 import DataSourceSnowflakeWorkerConfiguration
from .dataset_sink_pb2 import DatasetSinkWorkerConfiguration
from .delta_enclave_api_pb2 import (
    ChunkHeader,
    DataNoncePubkey,
    EncryptionHeader,
    Request,
    Response,
    VersionHeader,
)
from .gcg_pb2 import (
    CreateConfigurationCommitRequest,
    CreateConfigurationCommitResponse,
    CreateDataRoomRequest,
    CreateDataRoomResponse,
    DataRoomStatus,
    DriverTaskConfig,
    ExecuteComputeRequest,
    ExecuteComputeResponse,
    ExecuteDevelopmentComputeRequest,
    GcgRequest,
    GcgResponse,
    GenerateMergeApprovalSignatureRequest,
    GenerateMergeApprovalSignatureResponse,
    GetResultsRequest,
    GetResultsResponseChunk,
    GetResultsSizeRequest,
    JobStatusRequest,
    JobStatusResponse,
    MergeConfigurationCommitRequest,
    MergeConfigurationCommitResponse,
    NoopConfig,
    Pki,
    PublishDatasetToDataRoomRequest,
    PublishDatasetToDataRoomResponse,
    RemovePublishedDatasetRequest,
    RemovePublishedDatasetResponse,
    RetrieveAuditLogRequest,
    RetrieveAuditLogResponse,
    RetrieveConfigurationCommitApproversRequest,
    RetrieveConfigurationCommitRequest,
    RetrieveConfigurationCommitResponse,
    RetrieveCurrentDataRoomConfigurationRequest,
    RetrieveCurrentDataRoomConfigurationResponse,
    RetrieveDataRoomRequest,
    RetrieveDataRoomResponse,
    RetrieveDataRoomStatusRequest,
    RetrieveDataRoomStatusResponse,
    RetrievePublishedDatasetsRequest,
    RetrievePublishedDatasetsResponse,
    RetrieveUsedAirlockQuotaRequest,
    RetrieveUsedAirlockQuotaResponse,
    StaticContentConfig,
    TestDataset,
    UpdateDataRoomStatusRequest,
    UpdateDataRoomStatusResponse,
    UserAuth,
)
from .google_ad_manager_pb2 import GoogleAdManagerWorkerConfiguration
from .google_dv_360_sink_pb2 import GoogleDv360SinkWorkerConfiguration
from .identity_endorsement_pb2 import (
    DcrSecretEndorsementRequest,
    DcrSecretEndorsementResponse,
    EnclaveEndorsement,
    EnclaveEndorsements,
    EndorsementRequest,
    EndorsementResponse,
    PkiEndorsementRequest,
    PkiEndorsementResponse,
)
from .length_delimited import parse_length_delimited, serialize_length_delimited
from .meta_sink_pb2 import MetaSinkWorkerConfiguration
from .metering_pb2 import CreateDcrKind, CreateDcrPurpose, DcrMetadata
from .permutive_pb2 import PermutiveWorkerConfiguration
from .salesforce_pb2 import SalesforceWorkerConfiguration
from .synth_data_pb2 import Column, Mask, SyntheticDataConf



from typing import List


class MediaInsightFeatures:
    def __init__(self, features: List[str]) -> None:
        self.features = features

    def has_enable_lookalike(self) -> bool:
        return self._features_contain("ENABLE_LOOKALIKE")

    def has_enable_insights(self) -> bool:
        return self._features_contain("ENABLE_INSIGHTS")

    def has_enable_retargeting(self) -> bool:
        return self._features_contain("ENABLE_RETARGETING")

    def has_enable_rate_limiting_on_publish_dataset(self) -> bool:
        return self._features_contain("ENABLE_RATE_LIMITING_ON_PUBLISH_DATASET")

    def has_enable_model_performance_evaluation(self) -> bool:
        return self._features_contain("ENABLE_MODEL_PERFORMANCE_EVALUATION")

    def has_enable_timer_report(self) -> bool:
        return self._features_contain("ENABLE_TIMER_REPORT")

    def has_enable_noisy_output(self) -> bool:
        return self._features_contain("NOISY_OUTPUTS")

    def has_enable_advertiser_audience_download(self) -> bool:
        return self._features_contain("ENABLE_ADVERTISER_AUDIENCE_DOWNLOAD")

    def _features_contain(self, flag: str) -> bool:
        return flag in self.features



from .builder import MediaDcrBuilder
from .media import MediaDcr, MediaDcrDefinition
from .audience import Audience, ActivationType
from .version import MEDIA_DCR_SUPPORTED_VERSION


__pdoc__ = {
    "builder": False,
    "media": False,
}

__all__ = [
    "MediaDcrBuilder",
    "MediaDcr",
    "MediaDcrDefinition",
    "Audience",
    "ActivationType",
]



from __future__ import annotations
import base64
import uuid

from ..attestation import enclave_specifications
from typing_extensions import Self
from typing import Dict, List, Optional, TYPE_CHECKING, Tuple
from ..types import MATCHING_ID_INTERNAL_LOOKUP, MatchingId, EnclaveSpecification
from .media import MediaDcrDefinition
from decentriq_dcr_compiler import compiler
from decentriq_dcr_compiler.schemas import CreateMediaInsightsDcr
from ..proto import serialize_length_delimited
from .version import MEDIA_DCR_SUPPORTED_VERSION

if TYPE_CHECKING:
    from ..client import Client


class MediaDcrBuilder:
    """
    A helper class to build a Media DCR.
    """

    def __init__(
        self,
        *,
        client: Client,
        enclave_specs: Optional[Dict[str, EnclaveSpecification]] = None,
    ) -> None:
        """
        Initialise a Media DCR builder.

        **Parameters**:
        - `client`: A `Client` object that can be used to retrieve information about the platform.
        - `enclave_specs`: Determines the types of enclaves that will supported by this Data Clean Room.
            If not specified, the latest enclave specifications known to this
            SDK version will be used.
        """
        self.client = client
        self.enclave_specs = (
            enclave_specs if enclave_specs else enclave_specifications.latest()
        )
        self.name = None
        self.main_publisher_email = None
        self.additional_publisher_emails = []
        self.main_advertiser_email = None
        self.additional_advertiser_emails = []
        self.agency_emails = []
        self.observer_emails = []
        self.data_partner_emails = []
        self.enable_data_partner = False
        self.matching_id = None
        self.enable_insights = False
        self.enable_lookalike = False
        self.enable_retargeting = False
        self.hide_absolute_values_from_insights = False
        self.enable_advertiser_audience_download = False

    def with_name(self, name: str) -> Self:
        """
        Set the name of the Data Clean Room.

        **Parameters**:
        - `name`: Name to be used for the Data Clean Room.
        """
        self.name = name
        return self

    def with_publisher_emails(
        self, main: str, additional: Optional[List[str]] = None
    ) -> Self:
        """
        Set the publisher email addresses.

        **Parameters**:
        - `main`: The main publisher email address.
        - `additional`: Optional list of additional publisher email addresses.
        """
        self.main_publisher_email = main
        if additional is not None:
            self.additional_publisher_emails = additional
        return self

    def with_advertiser_emails(
        self, main: str, additional: Optional[List[str]] = None
    ) -> Self:
        """
        Set the advertiser email addresses.

        **Parameters**:
        - `main`: The main advertiser email address.
        - `additional`: Optional list of additional advertiser email addresses.
        """
        self.main_advertiser_email = main
        if additional is not None:
            self.additional_advertiser_emails = additional
        return self

    def with_agency_emails(self, emails: List[str]) -> Self:
        """
        Set the agency email addresses.

        **Parameters**:
        - `emails`: List of agency email addresses.
        """
        self.agency_emails = emails
        return self

    def with_observer_emails(self, emails: List[str]) -> Self:
        """
        Set the observer email addresses.

        **Parameters**:
        - `emails`: List of observer email addresses.
        """
        self.observer_emails = emails
        return self

    def with_data_partner_emails(self, emails: List[str]) -> Self:
        """
        Set the data partner email addresses.

        **Parameters**:
        - `emails`: List of data partner email addresses.
        """
        self.data_partner_emails = emails
        self.enable_data_partner = True
        return self

    def with_matching_id_format(self, matching_id: MatchingId) -> Self:
        """
        Set the matching ID format.

        **Parameters**:
        - `matching_id`: The type of matching ID to use.
        """
        self.matching_id = matching_id
        return self

    def with_insights(self) -> Self:
        """
        Enable the "insights" feature set.
        """
        self.enable_insights = True
        return self

    def with_lookalike(self) -> Self:
        """
        Enable the "lookalike" feature set.
        """
        self.enable_lookalike = True
        return self

    def with_retargeting(self) -> Self:
        """
        Enable the "retargeting" feature set.
        """
        self.enable_retargeting = True
        return self

    def with_advertiser_audience_download(self) -> Self:
        """
        Allow the advertiser to download the user ids for an audience.
        """
        self.enable_advertiser_audience_download = True
        return self

    def with_hide_absolute_values_from_insights(self) -> Self:
        """
        Ensure absolute values are not included in the output of the insights computation.
        """
        self.hide_absolute_values_from_insights = True
        return self

    def build(self) -> MediaDcrDefinition:
        """
        Build the Data Clean Room.

        In order to use the DCR, the output of this method should be passed to
        `client.publish_media_dcr`.
        """

        id = f"Media DCR {_generate_id()}"
        root_cert_pem = self.client.decentriq_ca_root_certificate.decode()
        (
            matching_id_format,
            matching_id_hashing_algorithm,
        ) = MATCHING_ID_INTERNAL_LOOKUP[self.matching_id]
        (driver_spec, python_spec) = self._get_media_insights_dcr_enclave_specs()

        # Dedupe the advertiser and publisher emails.
        # This ensures that the main email address only appears once even if it
        # exists in the addtional email addresses.
        advertiser_emails = list(set([self.main_advertiser_email] + self.additional_advertiser_emails))
        publisher_emails = list(set([self.main_publisher_email] + self.additional_publisher_emails))
        high_level = {
            MEDIA_DCR_SUPPORTED_VERSION: {
                "advertiserEmails": advertiser_emails,
                "agencyEmails": self.agency_emails,
                "authenticationRootCertificatePem": root_cert_pem,
                "driverEnclaveSpecification": {
                    "attestationProtoBase64": driver_spec.attestationProtoBase64,
                    "id": driver_spec.id,
                    "workerProtocol": driver_spec.workerProtocol,
                },
                "enableDebugMode": False,
                "enableInsights": self.enable_insights,
                "enableLookalike": self.enable_lookalike,
                "enableRetargeting": self.enable_retargeting,
                "hideAbsoluteValuesFromInsights": self.hide_absolute_values_from_insights,
                "enableAdvertiserAudienceDownload": self.enable_advertiser_audience_download,
                "hashMatchingIdWith": (
                    None
                    if matching_id_hashing_algorithm is None
                    else matching_id_hashing_algorithm.value
                ),
                "id": id,
                "mainAdvertiserEmail": self.main_advertiser_email,
                "mainPublisherEmail": self.main_publisher_email,
                "matchingIdFormat": matching_id_format.value,
                "modelEvaluation": {
                    "postScopeMerge": ["ROC_CURVE"],
                    "preScopeMerge": [],
                },
                "name": self.name,
                "observerEmails": self.observer_emails,
                "publisherEmails": publisher_emails,
                "pythonEnclaveSpecification": {
                    "attestationProtoBase64": python_spec.attestationProtoBase64,
                    "id": python_spec.id,
                    "workerProtocol": python_spec.workerProtocol,
                },
                "enableDataPartner": self.enable_data_partner,
                "dataPartnerEmails": self.data_partner_emails,
            }
        }
        media_insights_dcr = compiler.create_media_insights_dcr(
            CreateMediaInsightsDcr.model_validate(high_level)
        )
        dcr_definition = MediaDcrDefinition(
            name=self.name,
            high_level=media_insights_dcr.model_dump_json(by_alias=True),
            enclave_specs=self.enclave_specs,
        )
        return dcr_definition

    def _get_media_insights_dcr_enclave_specs(
        self,
    ) -> Tuple[compiler.EnclaveSpecification, compiler.EnclaveSpecification]:
        driver_spec = None
        python_spec = None
        for spec_id, spec in self.enclave_specs.items():
            spec_payload = {
                "attestationProtoBase64": base64.b64encode(
                    serialize_length_delimited(spec["proto"])
                ).decode(),
                "id": spec_id,
                "workerProtocol": spec["workerProtocols"][0],
            }
            if "decentriq.driver" in spec_id:
                driver_spec = compiler.EnclaveSpecification.model_validate(spec_payload)
            elif "decentriq.python-ml-worker" in spec_id:
                python_spec = compiler.EnclaveSpecification.model_validate(spec_payload)
        if driver_spec is None:
            raise Exception("No driver enclave spec found for the datalab")
        if python_spec is None:
            raise Exception("No python-ml-worker enclave spec found for the datalab")
        return (driver_spec, python_spec)


def _generate_id():
    return str(uuid.uuid4())



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: compute_post.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12\x63ompute_post.proto\x12\x0c\x63ompute_post\"1\n\x17PostWorkerConfiguration\x12\x16\n\x0euseMockBackend\x18\x01 \x01(\x08\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'compute_post_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_POSTWORKERCONFIGURATION']._serialized_start=36
  _globals['_POSTWORKERCONFIGURATION']._serialized_end=85
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

import io
import json
from typing import TYPE_CHECKING, Dict, Any, Optional, List

from ..storage import Key
from ..session import Session
from typing_extensions import Self
from ..attestation import enclave_specifications
from ..keychain import Keychain, KeychainEntry
from decentriq_dcr_compiler import compiler
from decentriq_dcr_compiler.schemas import (
    MediaInsightsRequest,
)
from .features import MediaInsightFeatures
from .publisher_computations import (
    OverlapInsightsComputation,
    OverlapStatisticsComputation,
    GetAudiencesForPublisherComputation,
    GetAudienceUserListComputation,
)

from .advertiser_computations import (
    GetAudiencesForAdvertiserComputation,
    GetAudienceUserListForAdvertiserComputation,
)
from .request import Request
from .audience import Audience
from ..types import EnclaveSpecification
from .version import MEDIA_DCR_SUPPORTED_VERSION, MEDIA_DCR_WRAPPER_SUPPORTED_VERSION

if TYPE_CHECKING:
    from ..client import Client

__docformat__ = "restructuredtext"


class MediaDcrDefinition:
    """
    Class representing a Media DCR Definition.
    """

    def __init__(
        self,
        name: str,
        high_level: Dict[str, Any],
        enclave_specs: Optional[Dict[str, EnclaveSpecification]] = None,
    ) -> None:
        self.name = name
        self._high_level = high_level
        self._enclave_specs = enclave_specs


class MediaDcr:
    """
    Class representing a Media DCR.
    """

    def __init__(
        self,
        dcr_id: str,
        high_level: Dict[str, Any],
        session: Session,
        *,
        client: Client,
    ) -> None:
        """
        Initialise a Media DCR.

        **Parameters**:
        - `dcr_id`: ID of the Media DCR.
        - `high_level`: High level representation of the Media DCR.
        - `session`: A `Session` object which can be used for communication with the enclave.
        - `client`: A `Client` object which can be used to perform operations such as uploading data
            and retrieving computation results.
        """
        self.client = client
        self.session = session
        self.high_level = high_level
        self.id = dcr_id
        self.features = _get_features(high_level)

    def retrieve_audit_log(self) -> str:
        """
        Retrieve the audit log.
        """
        return self.session.retrieve_audit_log(self.id).log.decode("utf-8")

    def stop(self):
        """
        Stop the Media DCR.
        """
        self.session.stop_data_room(self.id)

    def get_insights(self) -> Dict[str, Any]:
        """
        Get the insights.
        """
        if not self.features.has_enable_insights():
            raise Exception("This Media DCR does not support insights.")

        overlap_insights_computation = OverlapInsightsComputation(
            dcr_id=self.id, client=self.client, session=self.session
        )
        return overlap_insights_computation.run_and_get_results()

    def get_overlap_statistics(self) -> Dict[str, Any]:
        """
        Get the overlap statistics.
        """
        overlap_statistics_computation = OverlapStatisticsComputation(
            dcr_id=self.id, client=self.client, session=self.session
        )
        return overlap_statistics_computation.run_and_get_results()

    def get_audiences_for_publisher(self) -> Dict[str, Any]:
        """
        Get the audiences for the publisher.
        """
        if (
            not self.features.has_enable_lookalike()
            and not self.features.has_enable_retargeting()
        ):
            raise Exception(
                "Unable to retrieve audiences. Lookalike or Retargeting must be enabled."
            )

        get_audiences_for_publisher_computation = GetAudiencesForPublisherComputation(
            dcr_id=self.id, client=self.client, session=self.session
        )
        return get_audiences_for_publisher_computation.run_and_get_results()

    def get_audiences_for_advertiser(self) -> Dict[str, Any]:
        """
        Get the audiences for the advertiser.
        """
        if (
            not self.features.has_enable_lookalike()
            and not self.features.has_enable_retargeting()
        ):
            raise Exception(
                "Unable to retrieve audience. Lookalike or Retargeting must be enabled."
            )

        get_audiences_for_advertiser_computation = GetAudiencesForAdvertiserComputation(
            dcr_id=self.id, client=self.client, session=self.session
        )
        return get_audiences_for_advertiser_computation.run_and_get_results()

    def get_audience_user_list(
        self,
        activated_audience: Audience,
    ) -> List[str]:
        """
        Get the audience user list.
        """
        get_audience_user_list_computation = GetAudienceUserListComputation(
            dcr_id=self.id,
            audience=activated_audience,
            client=self.client,
            session=self.session,
        )
        return get_audience_user_list_computation.run_and_get_results()

    def get_audience_user_list_for_advertiser(
        self,
        activated_audience: Audience,
    ) -> List[str]:
        """
        Get the list of user ids for the given audience.
        This method is to be called by the advertiser and is different from `get_audience_user_list`
        in that it can be used to download user ids for audiences that have not been made available
        to the publisher.
        The Media DCR must have been created with the `enable_advertiser_audience_download` set to `true`,
        in order for this feature to work.
        """
        if not self.features.has_enable_advertiser_audience_download():
            raise Exception("This Media DCR does not support downloading audiences for advertisers.")
        computation = GetAudienceUserListForAdvertiserComputation(
            dcr_id=self.id,
            audience=activated_audience,
            client=self.client,
            session=self.session,
        )
        return computation.run_and_get_results()

    def provision_from_data_lab(self, data_lab_id: str, keychain: Keychain):
        """
        Provision the DataLab with the given ID to the Media DCR.

        **Parameters**:
        - `data_lab_id`: ID of the DataLab to provision to the Media DCR.
        - `keychain`: Keychain to use to provision datasets from the DataLab.
        """
        # First deprovision any existing datalabs before provisioning a new one.
        # This ensures that we don't get into issues with optional datasets not
        # being updated.
        self.deprovision_data_lab()

        # Check DataLab is validated
        data_lab = self.client.get_data_lab(data_lab_id)
        if not data_lab["isValidated"]:
            raise Exception("Cannot provision DataLab, not validated.")

        # Check compatibility
        hl_data_lab = data_lab["highLevelRepresentationAsString"]
        compatible = compiler.is_data_lab_compatible_with_media_insights_dcr_serialized(
            hl_data_lab, json.dumps(self.high_level)
        )
        if not compatible:
            raise Exception("DataLab is incompatible with Media DCR")

        # Provision datasets
        data_lab_datasets = self._get_data_lab_datasets_dict(data_lab)

        # Provision all existing Data Lab datasets to the DCR.
        for dataset_type, dataset in data_lab_datasets.items():
            if not dataset:
                # Dataset was not provisioned to the Data Lab.
                continue
            manifest_hash = dataset["manifestHash"]
            encryption_key = keychain.get("dataset_key", manifest_hash)
            if dataset_type == "MATCHING_DATA":
                request_key = "publishPublisherUsersDataset"
            elif dataset_type == "SEGMENTS_DATA":
                request_key = "publishSegmentsDataset"
            elif dataset_type == "DEMOGRAPHICS_DATA":
                request_key = "publishDemographicsDataset"
            elif dataset_type == "EMBEDDINGS_DATA":
                request_key = "publishEmbeddingsDataset"
            else:
                raise Exception(
                    f"Failed to provision Data Lab. Dataset type '{dataset_type}' unknown."
                )
            self._send_publish_dataset_request(
                request_key, manifest_hash, encryption_key
            )

        # Update DB.
        provisioned_data_lab_id = self.client._provision_data_lab_to_midcr(
            self.id, data_lab_id
        )["id"]
        if provisioned_data_lab_id != data_lab_id:
            raise Exception(
                f"Incorrectly provisioned DataLab {provisioned_data_lab_id}"
            )

    def _send_publish_dataset_request(
        self, request_key: str, manifest_hash: str, encryption_key: KeychainEntry
    ):
        request = MediaInsightsRequest.model_validate(
            {
                request_key: {
                    "dataRoomIdHex": self.id,
                    "datasetHashHex": manifest_hash,
                    "encryptionKeyHex": encryption_key.value.hex(),
                    "scopeIdHex": self.client._ensure_dcr_data_scope(self.id),
                },
            }
        )
        response = Request.send(request, self.session)
        if request_key not in response.model_dump_json():
            raise Exception(f'Failed to publish "{request_key}"')

    # This function should not throw on repeated calls. This allows it to be
    # called when provisioning also (where there may not yet be any data provisioned).
    def deprovision_data_lab(self):
        """
        Deprovision a DataLab from the Media DCR.
        """
        # Get a list of published datasets
        request = MediaInsightsRequest.model_validate(
            {
                "retrievePublishedDatasets": {
                    "dataRoomIdHex": self.id,
                },
            }
        )
        response = Request.send(request, self.session)
        datasets = response.model_dump()["retrievePublishedDatasets"]
        if datasets["publisherDatasetHashHex"]:
            request_key = "unpublishPublisherUsersDataset"
        elif datasets["segmentsDatasetHashHex"]:
            request_key = "unpublishSegmentsDataset"
        elif datasets["demographicsDatasetHashHex"]:
            request_key = "unpublishDemographicsDataset"
        elif datasets["embeddingsDatasetHashHex"]:
            request_key = "unpublishEmbeddingsDataset"
        else:
            request_key = None

        if request_key:
            self._send_unpublish_dataset_request(request_key)
            # Update DB.
            data_lab = self.client._deprovision_data_lab_from_midcr(self.id)
            if data_lab:
                raise Exception("DataLab should have been deprovisioned")

    def _send_unpublish_dataset_request(self, request_key: str):
        request = MediaInsightsRequest.model_validate(
            {
                request_key: {
                    "dataRoomIdHex": self.id,
                },
            }
        )
        response = Request.send(request, self.session)
        if request_key not in response.model_dump_json():
            raise Exception(f"Failed to unpublish {request_key}")

    def _get_data_lab_datasets_dict(self, data_lab: compiler.DataLab):
        datasets_dict = {}
        for dataset in data_lab["datasets"]:
            datasets_dict[dataset["name"]] = dataset["dataset"]
        return datasets_dict

    @classmethod
    def _from_existing(
        cls,
        dcr_id: str,
        *,
        client: Client,
        enclave_specs: Optional[List[EnclaveSpecification]] = None,
    ) -> Self:
        """
        Construct a Media DCR from an existing DCR with the given ID.

        **Parameters**:
        - `dcr_id`: ID of the Media DCR.
        - `client`: A `Client` object which can be used to perform operations such as uploading data
            and retrieving computation results.
        - `enclave_specs`: Determines the types of enclaves that are supported by this Data Clean Room.
            If not specified, the latest enclave specifications will be used.
        """
        specs = enclave_specs if enclave_specs else enclave_specifications.all()
        specs_dict = {spec["name"]: spec for spec in specs}
        existing_data_room_description = client.get_data_room_description(
            dcr_id, enclave_specs=specs_dict
        )
        if not existing_data_room_description:
            raise Exception(
                f"Unable to retrieve data room description for data room with ID {dcr_id}"
            )
        session = client.create_session_from_data_room_description(
            existing_data_room_description, specs
        )
        existing_dcr = session.retrieve_data_room(dcr_id)
        high_level = json.loads(existing_dcr.highLevelRepresentation.decode())

        dcr = cls(dcr_id=dcr_id, session=session, high_level=high_level, client=client)
        return dcr

    def activate_audience(
        self,
        audiences: List[Audience],
    ):
        """
        Activate the list of audiences, making it accessible to the Publisher if they are published.
        The Publisher will be able to download the user ids belonging to all the audiences, for which the is_published flag is set to True.

        **Parameters**:
        - `audiences`: List of audiences that should be made available to the Publisher.
        """
        activate_audience_list_json = json.dumps(
            {"activated_audiences": [audience.as_dict() for audience in audiences]}
        )
        key = Key()
        manifest_hash = self.client.upload_dataset(
            io.BytesIO(activate_audience_list_json.encode()),
            key,
            "audiences.json",
        )
        self.session.publish_dataset(
            self.id,
            manifest_hash,
            "activated_audiences.json",
            key,
        )

    def participants(self) -> Dict[str, Any]:
        """
        Retrieve the participants of the Media DCR.
        This returns a dictionary of roles (keys) mapped to participants (email addresses).
        """
        dcr = self.high_level[MEDIA_DCR_WRAPPER_SUPPORTED_VERSION]
        compute_keys_list = list(dcr["compute"].keys())
        if len(compute_keys_list) != 1:
            raise Exception(
                f"Unable to extract Media DCR version. Expected a single top-level property indicating the DCR version."
            )

        compute_version = compute_keys_list[0]
        compute = dcr["compute"][compute_version]
        return {
            "publisher": compute["publisherEmails"],
            "advertiser": compute["advertiserEmails"],
            "observer": compute["observerEmails"],
            "agency": compute["agencyEmails"],
        }


def _get_features(high_level: Dict[str, Any]) -> MediaInsightFeatures:
    features = compiler.get_media_insights_dcr_features_serialized(
        json.dumps(high_level)
    )
    return MediaInsightFeatures(features)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class ImportRole(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ImportRole = ImportRole

@typing_extensions.final
class ExportRole(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INPUT_FIELD_NUMBER: builtins.int
    @property
    def input(self) -> global___SinkInput: ...
    def __init__(
        self,
        *,
        input: global___SinkInput | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["input", b"input"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["input", b"input"]) -> None: ...

global___ExportRole = ExportRole

@typing_extensions.final
class SingleFile(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    name: builtins.str
    def __init__(
        self,
        *,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name"]) -> None: ...

global___SingleFile = SingleFile

@typing_extensions.final
class RawFile(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___RawFile = RawFile

@typing_extensions.final
class ZipFile(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SINGLEFILE_FIELD_NUMBER: builtins.int
    @property
    def singleFile(self) -> global___SingleFile: ...
    def __init__(
        self,
        *,
        singleFile: global___SingleFile | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["selection", b"selection", "singleFile", b"singleFile"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["selection", b"selection", "singleFile", b"singleFile"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["selection", b"selection"]) -> typing_extensions.Literal["singleFile"] | None: ...

global___ZipFile = ZipFile

@typing_extensions.final
class SinkInput(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DEPENDENCY_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    RAW_FIELD_NUMBER: builtins.int
    ZIP_FIELD_NUMBER: builtins.int
    dependency: builtins.str
    name: builtins.str
    @property
    def raw(self) -> global___RawFile: ...
    @property
    def zip(self) -> global___ZipFile: ...
    def __init__(
        self,
        *,
        dependency: builtins.str = ...,
        name: builtins.str = ...,
        raw: global___RawFile | None = ...,
        zip: global___ZipFile | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["file", b"file", "raw", b"raw", "zip", b"zip"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["dependency", b"dependency", "file", b"file", "name", b"name", "raw", b"raw", "zip", b"zip"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["file", b"file"]) -> typing_extensions.Literal["raw", "zip"] | None: ...

global___SinkInput = SinkInput

@typing_extensions.final
class PermutiveWorkerConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CREDENTIALSDEPENDENCY_FIELD_NUMBER: builtins.int
    IMPORTROLE_FIELD_NUMBER: builtins.int
    EXPORTROLE_FIELD_NUMBER: builtins.int
    IMPORTID_FIELD_NUMBER: builtins.int
    SEGMENTNAME_FIELD_NUMBER: builtins.int
    SEGMENTCODE_FIELD_NUMBER: builtins.int
    INPUTHASHEADERS_FIELD_NUMBER: builtins.int
    credentialsDependency: builtins.str
    @property
    def importRole(self) -> global___ImportRole: ...
    @property
    def exportRole(self) -> global___ExportRole: ...
    importId: builtins.str
    """
    importId, segmentName and segmentCode are required when
    creating a segment for an import via the Permutive API. 
    These fields are not well documented by Permutive and as
    such they are required to be provided by the client.
    """
    segmentName: builtins.str
    segmentCode: builtins.str
    inputHasHeaders: builtins.bool
    def __init__(
        self,
        *,
        credentialsDependency: builtins.str = ...,
        importRole: global___ImportRole | None = ...,
        exportRole: global___ExportRole | None = ...,
        importId: builtins.str = ...,
        segmentName: builtins.str = ...,
        segmentCode: builtins.str = ...,
        inputHasHeaders: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["exportRole", b"exportRole", "importRole", b"importRole", "role", b"role"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["credentialsDependency", b"credentialsDependency", "exportRole", b"exportRole", "importId", b"importId", "importRole", b"importRole", "inputHasHeaders", b"inputHasHeaders", "role", b"role", "segmentCode", b"segmentCode", "segmentName", b"segmentName"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["role", b"role"]) -> typing_extensions.Literal["importRole", "exportRole"] | None: ...

global___PermutiveWorkerConfiguration = PermutiveWorkerConfiguration



from __future__ import annotations

import io
import json
from typing import Dict, Any, List, Optional, TYPE_CHECKING
import zipfile

from ..session import Session
from .computations import Computation
from ..types import JobId
from decentriq_dcr_compiler.schemas import MediaInsightsRequest
from .audience import Audience

if TYPE_CHECKING:
    from ..client import Client


class OverlapInsightsComputation(Computation):
    def __init__(self, dcr_id: str, client: Client, session: Session) -> None:
        super().__init__(dcr_id=dcr_id, client=client, session=session)

    def node_id(self) -> str:
        return "compute_insights"

    def run(self) -> None:
        """
        Run the computation.
        """
        super().run(request_type="computeInsights")

    def get_results(
        self, interval: int = 5, timeout: Optional[int] = None
    ) -> Dict[str, Any]:
        result = super().get_results_str_from_zip(
            "segments.json", interval=interval, timeout=timeout
        )
        return json.loads(result)


class OverlapStatisticsComputation(Computation):
    def __init__(self, dcr_id: str, client: Client, session: Session) -> None:
        super().__init__(dcr_id=dcr_id, client=client, session=session)

    def node_id(self) -> str:
        return "compute_overlap_statistics"

    def run(self) -> None:
        """
        Run the computation.
        """
        super().run(request_type="computeOverlapStatistics")

    def get_results(
        self, interval: int = 5, timeout: Optional[int] = None
    ) -> Dict[str, Any]:
        result = super().get_results_str_from_zip(
            "overlap.json", interval=interval, timeout=timeout
        )
        return json.loads(result)


class GetAudiencesForPublisherComputation(Computation):
    def __init__(self, dcr_id: str, client: Client, session: Session) -> None:
        super().__init__(dcr_id=dcr_id, client=client, session=session)

    def node_id(self) -> str:
        return "get_audiences_for_publisher"

    def run(self) -> None:
        """
        Run the computation.
        """
        super().run(request_type="getAudiencesForPublisher")

    def get_results(
        self, interval: int = 5, timeout: Optional[int] = None
    ) -> Dict[str, Any]:
        result = super().get_results_str_from_zip(
            "activated_audiences.json", interval=interval, timeout=timeout
        )
        return json.loads(result)


class GetAudienceUserListComputation(Computation):
    def __init__(
        self,
        dcr_id: str,
        audience: Audience,
        client: Client,
        session: Session,
    ) -> None:
        super().__init__(dcr_id=dcr_id, client=client, session=session)
        self.audience = audience

    def node_id(self) -> str:
        return "get_audience_user_list"

    def run(self) -> None:
        """
        Run the computation.
        """
        request = MediaInsightsRequest.model_validate(
            {
                "getAudienceUserList": {
                    "dataRoomIdHex": self.dcr_id,
                    "scopeIdHex": self.client._ensure_dcr_data_scope(self.dcr_id),
                    "requestedAudience": {
                        "activation_type": self.audience.activation_type,
                        "audience_type": self.audience.audience_type,
                        "reach": self.audience.reach,
                    },
                },
            }
        )
        response = self.send_request(request).model_dump()
        self.job_id = JobId(
            job_id=response["getAudienceUserList"]["jobIdHex"],
            compute_node_id=self.node_id(),
        )

    def get_results(
        self, interval: int = 5, timeout: Optional[int] = None
    ) -> List[str]:
        result = super().get_results_str_from_zip(
            "audience_users.csv", interval=interval, timeout=timeout
        )
        return [line for line in result.split("\n") if line]
